#define _XOPEN_SOURCE
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <sys/sysinfo.h>

#include "fast-boot.h"

/**
 * Get time of system startup.
 */
static time_t
get_system_boot_time ()
{
  struct sysinfo info;
  time_t cur_time = 0;
  time_t boot_time = 0;
  struct tm *ptm = NULL;

  if (sysinfo(&info)) {
    return -1;
  }

  time(&cur_time);
  boot_time = cur_time - info.uptime;

  return boot_time;
}

/**
 * Get the last line of /var/log/dpkg.log.
 */
static char *
get_dpkg_log_file_last_line ()
{
  FILE* fp = fopen("/var/log/dpkg.log", "r");
  char a[512];

  while(!feof(fp)) {
    fgets(a,sizeof(a),fp);
  }
  fclose(fp);

  return g_strdup (a);
}

/**
 * Execute system command, return execution status.
 * And save the last line of the execution result in ret_msg.
 * if Exe Success, will be return 0.
 */
int 
fastboot_system (const char *cmd, char *ret_msg, int msg_len)
{
  FILE *fp;

  if (cmd == NULL || ret_msg == NULL || msg_len < 0)
  {
    return 1;
  }

  fp = popen (cmd, "r");
  if (fp == NULL)
  {
    return 2;
  }
  else
  {
    memset (ret_msg, 0, msg_len);

    /* get the last line. */
    while (fgets (ret_msg, msg_len, fp));
    pclose (fp);

    ret_msg[strlen (ret_msg) - 1] = '\0';

    return 0;
  }
}

/**
 * Check the status of software update after system startup.
 * if software was updated, return TRUE;
 * if not, return FALSE.
 */
gboolean
fastboot_software_update (void)
{
  time_t dpkg_update_time;
  struct tm log_tm;

  char *last_line = get_dpkg_log_file_last_line ();

  if (last_line) {
    char **strv_tmp = g_strsplit (last_line, " ", 3);
    char update_time[20] = {0, };

    g_free (last_line);

    if (strv_tmp[0] && strv_tmp[1]) {
      sprintf (update_time, "%s %s", strv_tmp[0], strv_tmp[1]);
      strptime (update_time, "%Y-%m-%d %H:%M:%S", &log_tm);
      dpkg_update_time = mktime (&log_tm);

      time_t system_boot_time = get_system_boot_time ();
      if ((system_boot_time != -1) && (system_boot_time < dpkg_update_time)) {
        g_strfreev (strv_tmp);
        return TRUE;
      }
    }

    g_strfreev (strv_tmp);
  }

  return FALSE;
}
